Package: diversitree
Version: 0.10-1
Title: Comparative 'Phylogenetic' Analyses of Diversification
Authors@R: c(person("Richard G.", "FitzJohn", role = c("aut", "cre"),
    email = "rich.fitzjohn@gmail.com"),
    person("Emma", "Goldberg", role = "aut"),
    person("Karen", "Magnuson-Ford", role = "aut"),
    person("Roger", "Sidje", role = "aut"))
Depends: R (>= 2.10), methods, ape
Imports: deSolve (>= 1.7), graphics, grDevices, stats, subplex, Rcpp
        (>= 0.10.0)
Suggests: numDeriv, minqa, lubridate, expm, caper, geiger
LinkingTo: Rcpp
RcppModules: diversitree
SystemRequirements: fftw3 (>= 3.1.2), gsl (>= 1.15)
Description: Contains a number of comparative 'phylogenetic' methods,
  mostly focusing on analysing diversification and character
  evolution.  Contains implementations of 'BiSSE' (Binary State
  'Speciation' and Extinction) and its unresolved tree extensions,
  'MuSSE' (Multiple State 'Speciation' and Extinction), 'QuaSSE',
  'GeoSSE', and 'BiSSE-ness' Other included methods include Markov
  models of discrete and continuous trait evolution and constant rate
  'speciation' and extinction.
License: GPL (>= 2)
URL: https://www.zoology.ubc.ca/prog/diversitree/
NeedsCompilation: yes
Packaged: 2024-09-30 17:34:00 UTC; rich
Author: Richard G. FitzJohn [aut, cre],
  Emma Goldberg [aut],
  Karen Magnuson-Ford [aut],
  Roger Sidje [aut]
Maintainer: Richard G. FitzJohn <rich.fitzjohn@gmail.com>
Repository: CRAN
Date/Publication: 2024-10-02 07:10:01 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 16:20:12 UTC; unix
Archs: diversitree.so.dSYM
